@echo off
setlocal EnableDelayedExpansion

rem Algebraic to RPN expressions converter
rem Reference: https://en.wikipedia.org/wiki/Shunting_yard_algorithm
rem Antonio Perez Ayala, 2023/01

cls

if /I "%RegionCode%" equ "LA" (
for /F "tokens=2 delims=:" %%a in ('chcp') do set "CP=%%a"
chcp 65001 > NUL
echo Convertidor de Expresiones Algebraicas a Notación Polaca Inversa (RPN^)
echo/
echo Ingrese expresiones algebraicas separando *cada elemento* por uno o más
echo espacios, y separando las funciones de sus paréntesis izquierdos.
echo/
echo Los números no pueden empezar con signo. No se permiten operaciones unarias.
echo/
echo Algunos ejemplos:
chcp %CP% > NUL
) else (
echo Algebraic Expression to Reverse Polish Notation (RPN^) converter
echo/
echo Enter algebraic expressions separating *each element* by one or more spaces,
echo and separating function names from its left parentheses.
echo/
echo Numbers can not start with minus sign and unary operators are not allowed.
echo/
echo Some examples:
)
echo    ( 4 + 5 ) / ( 6 + 7 )
echo    3 + 4 * 2 / ( 1 - 5 ) ** 2 ** 3
echo    4 + sin ( 3.1416 * 8 ) * cos ( 3.1416 * 8 )
echo    sqrt ( 35 * 35 + 44 * 44 )
echo    sin ( max ( 2 , 3 ) / 3 * 3.1416 )


rem Initialize operators precedences
set "prec[(]=1"
set "prec[+]=2"
set "prec[-]=2"
set "prec[*]=3"
set "prec[/]=3"
set "prec[**]=4"
rem Operators with right-to-left associativity
set "left=|**|"
set "digits=0123456789"

:nextAlgebraic
echo/
echo/
set "algebraic="
set /P "algebraic=Alg: "
if not defined algebraic goto :EOF

set /P "=RPN: " < NUL
set "operStack="
set "tos="

:nextToken
for /F "tokens=1*" %%a in ("%algebraic%") do set "token=%%a" & set "algebraic=%%b"

if "%token%" equ "," goto nextToken &	rem Ignore commas
if "%token%" equ "(" (			rem Left paren: insert it in oper stack
   set "operStack=( %operStack%"
) else if "%token%" equ ")" (		rem Right paren: extract operators until "("
   call :ExtractLeftParen
) else if defined prec[%token%] (	rem Token is an operator: insert in its stack
   call :InsertOperator
) else if "!digits:%token:~0,1%=!" neq "%digits%" (  rem Token is a number: output it
   set /P "=%token% " < NUL
) else (				rem Token is a function: insert it in oper stack
   set "operStack=%token% ( %operStack%"
   for /F "tokens=1*" %%a in ("%algebraic%") do (
      if "%%a" equ "(" (set "algebraic=%%b") else set "algebraic="
   )
   if not defined algebraic echo/& echo Error: function "%token%" bad written & set "operStack="
)

if defined algebraic goto nextToken

rem At end: pop out all operators left in its stack
if defined operStack if "!operStack:(=!" neq "!operStack!" echo/& echo Error: Left paren with NO closing right paren
echo/%operStack%
goto nextAlgebraic


:ExtractLeftParen

rem Extract previous operators until Left paren
if not defined operStack (
   echo/
   echo Error: Right paren with NO previous left paren
   set "algebraic="
   exit /B
)
for /F "tokens=1*" %%a in ("%operStack%") do (
   set "operStack=%%b"
   if "%%a" neq "(" set /P "=%%a " < NUL & goto ExtractLeftParen
)
exit /B


:InsertOperator

rem Extract previous operators with greater or equal precedence
rem or only with greater precedence if operator is associative to left
for /F "tokens=1*" %%a in ("%operStack%") do (
   set "tos=%%a"
   set "rest=%%b"
)
if !prec[%tos%]! geq !prec[%token%]! (
   if "!left:|%token%|=|!" neq "%left%" if !prec[%tos%]! equ !prec[%token%]! goto insertNew
   set /P "=%tos% " < NUL
   set "operStack=%rest%"
   if defined operStack goto InsertOperator
)
:insertNew
rem Insert new operator in the stack
set "operStack=%token% %operStack%"
exit /B

