@echo off
setlocal DisableDelayedExpansion

rem printf.exe Example: Two-variables statistical calculations.
rem FPU-only operations version
rem ===========================
rem Antonio Perez Ayala

rem Create a standard data file
(
echo  0.00	4.63
echo 20.00	5.78
echo 40.00	6.61
echo 60.00	7.21
echo 80.00	7.78
echo/
echo 65.00
echo 70.00
echo 75.00
) > "HP-15C Page 49 example.txt"

if "%~1" neq "/?" goto begin

echo Two-variables statistical calculations
echo/
echo     "%~0" textFile.txt
echo/
echo The textFile.txt must contain a series of X-Y value pairs terminated by an
echo empty line and followed by lines with X values used for Linear Estimations
echo/
echo The included example was taken from page 49 of HP-15C Owner's Handbook at
echo http://h10032.www1.hp.com/ctg/Manual/c03030589.pdf#page=49
echo/
type "HP-15C Page 49 example.txt"
echo/
echo/
choice /M "--> Do you want to open the HP-15C handbook now "
if %errorlevel% equ 1 explorer "http://h10032.www1.hp.com/ctg/Manual/c03030589.pdf#page=49"
goto :EOF


Emulating the operation of S+ key and statistical calculations in HP-15C calculator

Statistics storage registers used in HP-15C

R2	n	Number of data points accumulated.
R3	Sx	Summation of x-values.
R4	Sx^2	Summation of squares of x-values.
R5	Sy	Summation of y-values.
R6	Sy^2	Summation of squares of y-values.
R7	Sx*y	Summation of products of x- and y-values.

Statistical formulae
Reference: Appendix A of HP-15C Owner's Handbook at #page=206

R8	M = R2*R4 - R3^2
R9	N = R2*R6 - R5^2
R0	P = R2*R7 - R3*R5

Mean (average):
  Mx = R3 / R2
  My = R5 / R2

Standard Deviation:
  Sx = SQRT( R8 / (R2*(R2-1)) )
  Sy = SQRT( R9 / (R2*(R2-1)) )

Coefficients for Linear Regression: y = A*x + B
  A = R0 / R8
  B = (R8*R5 - R0*R3) / (R2*R8)

After Linear Regression Coefficients are calculated,
they are stored at R.1=A and R.2=B

Correlation coefficient:
  r = R0 / SQRT(R8*R9)

Linear Estimation:
  Direct formula:  y ~= A*x + B  =  R.1*x + R.2
  HP's formula:    y ~= ( R8*R5 + R0*(R2*x - R3) ) / (R2*R8)

Direct formula values are equivalent to HP's formula:
y =    A   *x   +   B
y = (R0/R8)*x   +   (R8*R5 - R0*R3) / (R2*R8)
  = (R0/R8)*(R2*R8)*x / (R2*R8)   +   (R8*R5 - R0*R3) / (R2*R8)
  = (  (R0/R8)*(R2*R8)*x + (R8*R5 - R0*R3)  ) / (R2*R8)
  = (  (R0)*(R2)*x + (R8*R5 - R0*R3)  ) / (R2*R8)
  = (  (R0*R2)*x + R8*R5 - R0*R3  ) / (R2*R8)
  = (  R8*R5 + (R0*R2)*x - R0*R3  ) / (R2*R8)
  = (  R8*R5 + R0*(R2*x - R3)  ) / (R2*R8)


Statistics storage registers used in HP-15C
in FPU only version, six FPU stack registers are used instead of R2..R7 storage registers

R2	B=	n	Number of data points accumulated.
R3	A=	Sx	Summation of x-values.
R4	T=	Sx^2	Summation of squares of x-values.
R5	Z=	Sy	Summation of y-values.
R6	Y=	Sy^2	Summation of squares of y-values.
R7	X=	Sxy	Summation of products of x- and y-values.

:begin

set "file=HP-15C Page 49 example.txt"
if "%~1" neq "" set "file=%~1"
if not exist "%file%" echo File not found: "%file%" & goto :EOF

echo Input file "%file%":
type "%file%"
echo/

rem Didactic form
< "%file%" printf ""  /* no base format */ ^
   /* Part 0: Enter  B=0  A=0  T=0  Z=0  Y=0  X=0		*/ ^
   ZERO ZERO ZERO ZERO ZERO ZERO REMOVE          /* in FPU only	*/ ^
   /* Part 1: Populate statistics values in FPU only registers	*/ ^
   (		/*  B=0  A=0  T=0    Z=0  Y=0    X=0    ! WHILE read line	*/ ^
      80	/*                                      !    80   (input len)	*/ ^
      IN	/*                                      !    "x.xx y.yy" len	*/ ^
      ==0? ;	/*                                      ! line empty?  break	*/ ^
   /" < /"	/*                                      !   "x.xx y.yy"		*/ ^
      atof1	/*  D=0  C=0  B=0    A=0  T=0    Z=0    !    x. y. 2		*/ ^
   /" < /"	/*                                      !    x. y.		*/ ^
      STO:++8	/*  D++                                 !       Accum n		*/ ^
      STO:+5	/*                   A+=y               !       Accum Sy	*/ ^
      STO9	/*                                      !       R9 = y		*/ ^
      Sqr	/*                                      !    x. y^2		*/ ^
      STO:+4	/*                        T+=y^2        !       Accum Sy^2	*/ ^
      DROP	/*  C=n  B=0  A=0    T=y  Z=y^2  Y=0    !    x.			*/ ^
      STO:+6	/*       B+=x                           !       Accum Sx	*/ ^
      DUP	/*  D=n  C=x  B=0    A=y  T=y^2  Z=0    !    x. x.		*/ ^
      Sqr	/*                                      !    x. x^2		*/ ^
      STO:+6	/*            B+=x^2                    !       Accum Sx^2	*/ ^
      DROP	/*  C=n  B=x  A=x^2  T=y  Z=y^2  Y=0    !    x.			*/ ^
      RCL*9	/*                                      !    x*y		*/ ^
      STO:+2	/*  C=n  B=x  A=x^2  T=y  Z=y^2  Y+=x*y !       Accum Sx*y	*/ ^
      DROP	/*  B=n  A=x  T=x^2  Z=y  Y=y^2  X=x*y  !       Empty params	*/ ^
   :		/* REPEAT		*/ ^
   )		/* ENDWHILE		*/ ^
   REMOVE	/* Remove params	*/ ^
   FMT{ "Statistics registers:\n" OUT FMT}:	        /* Header	*/ ^
   FMT{ "R2 = B\tn:\t\t%%.0f\nR3 = A\tSum x:\t\t%%.2f\nR4 = T\tSum x^2:\t%%.2f\nR5 = Z\tSum y:\t\t%%.2f\nR6 = Y\tSum y^2:\t%%.2f\nR7 = X\tSum x*y:\t%%.2f\n\n"	/* Format */ ^
        LOAD* OUT FMT}:					/* Load:  n  Sx  Sx^2  Sy  Sy^2  Sx*y  from FPU stack and output */ ^
   /* Storage registers in original formulae are now mapped to these FPU stack registers: */ ^
   /* R2 = B, R3 = A, R4 = T, R5 = Z, R6 = Y, R7 = X		*/ ^
   /* Part 2: Calculate statistics values			*/ ^
   /*    R8  =  M = R2*R4 - R3^2				*/ ^
   /*    R9  =  N = R2*R6 - R5^2				*/ ^
   /*    R0  =  P = R2*R7 - R3*R5				*/ ^
   RCL:6 RCL:*5 RCL:6 STO:*1 STO:-2 DROP:1 STO8 DROP:1	/* M	*/ ^
   RCL:6 RCL:*3 RCL:4 STO:*1 STO:-2 DROP:1 STO9 DROP:1	/* N	*/ ^
   RCL:6 RCL:*2 RCL:6 RCL:*5 STO:-2 DROP:1 STO0 DROP:1	/* P	*/ ^
   /* Part 3: Calculate and output results			*/ ^
   /*    Mean:  Mx = R3 / R2,  My = R5 / R2			*/ ^
   FMT{ "Mean (average):\t\tMx=%%.2f  My=%%.2f\n"  RCL:5 RCL:/7 LOAD:1 RCL:3 RCL:/7 LOAD:1  OUT FMT}:	/* Mean */ ^
   /*    Standard Deviation:  Sx = SQRT( R8 / (R2*(R2-1)) ), Sy = SQRT( R9 / (R2*(R2-1)) )  */ ^
   FMT{ "Standard Deviation:\tSx=%%.2f  Sy=%%.2f\n"	/* Standard Deviation		    */ ^
        RCL:6 RCL:-- RCL:*7 RCL8 RCL:/2 Sqrt DROP:1  RCL9 RCL:/2 Sqrt DROP:1 DROP:1  OUT FMT}:	/* Sx, Sy */ ^
   DROP:1 DROP:1 /* Eliminate R7 = X and R6 = Y, so from this point on: */ ^
   /* R2 = T, R3 = Z, R4 = Y, R5 = X					*/ ^
   /* Coefficients for Linear Regression:  y = A*x + B			*/ ^
   /*    R.1 = A = R0 / R8,   R.2 = B = (R8*R5 - R0*R3) / (R2*R8)	*/ ^
   FMT{ "Linear Regression:\ty = %%.2f*x + %%.2f\n"		/* L.R.	*/ ^
        RCL0 RCL/8   RCL8 RCL:*3 RCL0 RCL:*6 Sub RCL8 RCL:*7 Div  OUT FMT}:	/* A*x+B  */ ^
   /* At this point: R2 = B, R3 = A, R4 = T, R5 = Z, A = Y, B = X	*/ ^
   /* Correlation coefficient:  r = R0 / SQRT(R8*R9)	*/ ^
   FMT{ "Correlation coeff:\tr = %%.2f\n\n"  RCL0 RCL8 RCL*9 Sqrt Div DROP:1  OUT FMT}:		/* r */ ^
   /* Part 3: Linear Estimations loop */ ^
   FMT{ "Linear Estimations:\n" OUT FMT}:	/* Header */ ^
   FMT{ "%%.2f   %%.2f\n"			/* Data   */ ^
   (		/* WHILE read line	*/ ^
      80	/*    80   (input len)	*/ ^
      IN?	/*    "x.xx" len	*/ ^
   /" < /"	/*    "x.xx"		*/ ^
      atof	/*    x.		*/ ^
      DUP	/*    x. x.		*/ ^
      /* FPU Stack here: R2 = D, R3 = C, R4 = B, R5 = A, A = T, B = Z  !  x. x.	*/ ^
      RCL:*4 RCL:+3  /* y=A*x+B         */ ^
      REMOVE	/*    remove x. x. from printf.exe parameters	*/ ^
      LOAD:2 LOAD:1	/* pass x and y from FPU to parameters	*/ ^
      OUT	/*      show estimation	*/ ^
      REMOVE	/*      remove parameters*/ ^
   :		/* REPEAT		*/ ^
   )		/* ENDHILE		*/

pause
