@if (@CodeSection == @Batch) @then

@echo off
setlocal EnableDelayedExpansion

if "%1" neq "" goto %1

rem Turtle Graphics implementation for printf.exe
rem Antonio Perez Ayala  July/2024

rem Introduction to Turtle Graphics in Logo:
rem http://www.logointerpreter.com/logo-reference/

rem Turtle Graphics reference at Berkeley Logo Reference Manual:
rem https://people.eecs.berkeley.edu/~bh/v2ch14/manual.html

title Turtle Graphics in printf.exe
set "openSite="

:begin
cls
echo/

CHCP 65001 > NUL

=IF= /I "%RegionCode%" NEQ "LA" GOTO :ELSE
echo          Gráficas de la Tortuga en printf.exe, por Antonio Pérez Ayala
echo/
echo Estas gráficas se trazan por una "Tortuga" que se mueve por la pantalla
echo arrastrando un "lápiz" amarrado a su cola. Usted le da órdenes a la tortuga
echo para que camine hacia adelante o hacia atrás y para que gire a la derecha o a
echo la izquierda, por lo que el "lápiz" va trazando líneas en la ventana gráfica.
echo/
echo ¿Cómo desea utilizar a la Tortuga?
echo/
echo    1. En órdenes interactivas para practicar el trazo de líneas.
echo    2. Trazando algunas figuras predefinidas.
echo/
choice /N /C 120 /M "Seleccione la opción deseada, o 0 para terminar: "
goto Option_%errorlevel%
:ELSE
echo           Turtle Graphics in printf.exe, by Antonio Perez Ayala
echo/
echo These graphics are drawn by a "Turtle" that moves around the screen dragging
echo a "pen" attached to its tail. You give the turtle commands to walk forward or
echo backward and to turn left or right, so the "pen" draw lines on the window.
echo/
echo How do you want to use the Turtle?
echo/
echo    1. In interactive commands to practice drawing lines.
echo    2. Drawing some predefined figures.
echo/
choice /N /C 120 /M "Select the desired option, or 0 to end: "
goto Option_%errorlevel%
:=ENDIF=

====================================

:Option_1	Interactive commands

=IF= /I "%RegionCode%" NEQ "LA" GOTO :ELSE
:helpLA
set "break="
cls
echo/
echo          Gráficas de la Tortuga en printf.exe, por Antonio Pérez Ayala
echo/
:help2LA

CHCP 65001 > NUL

echo pixels   avanza     (av)   Avanza la Tortuga pixels hacia adelante.
echo pixels   retrocede  (re)   Retrocede la Tortuga pixels hacia atrás.
echo grados.  derecha    (de)   Gira la Tortuga grados a la derecha (¡valor PF^^^!).
echo grados.  izquierda  (iz)   Gira la Tortuga grados a la izquierda (¡valor PF^^^!).
echo/
echo          subeLapiz  (sl)   Sube el lápiz: el movimiento ya no traza líneas.
echo          bajaLapiz  (bl)   Baja el lápiz: el movimiento traza líneas de nuevo.
echo/
echo color    ponColor   (pc)   Pone el color del lápiz (de 0 a 15).
echo color    ponFondo   (pf)   Pone el color del fondo de la ventana (de 0 a 15).
echo/
echo          centro            Mueve la Tortuga al centro, apuntando hacia arriba.
echo          borraPantalla(bp) Borra la ventana gráfica y reinicializa los valores.
echo/
echo Algunos ejemplos sencillos:
echo    100 avanza   45. derecha   13 ponColor   50 avanza
echo    90. izquierda   5 (  100 avanza   72. derecha   --? : ) 
echo    12 ( 190 avanza  150. derecha  --? : )
echo    0 ( ^> avanza  30. derecha  ++ 100 ^<=x? ^< : )
echo/
echo -^> Recuerde que los grados para los giros *DEBEN LLEVAR* punto decimal ^<-
echo/
echo Para terminar el dibujo, teclee: fin
%break%
echo Para obtener ayuda, teclee: ayuda  (puede teclear help para ayuda en inglés)
set "Msg[Title]=Ingrese el título de una nueva ventana (o ayuda): "
set "Msg[Line]=Seleccione el grueso de la línea "
set "Msg[Window]=Ingrese los tamaños X,Y de la ventana en pixels [^!sizes^!]: "
set "Msg[Window2]=-^^> Se sugiere mover la ventana "^^!title^^!" a un área visible ^^<-"
set "Msg[End]=Para cerrar la ventana, selecciónela y oprima una tecla"
set "Msg[Ready]=Listo..."
=GOTO= :ENDIF
:ELSE
:help
set "break="
cls
echo/
echo           Turtle Graphics in printf.exe, by Antonio Perez Ayala
echo/
:help2
echo pixels    forward  (fd)   Move the Turtle pixels forward.
echo pixels    back     (bk)   Move the Turtle pixels backward.
echo degrees.  right    (rt)   Rotate the Turtle degrees clockwise (FP value^^^!).
echo degrees.  left     (lt)   Rotate the Turtle degrees counterclockwise (FP^^^!).
echo/
echo           penUp    (pu)   Put pen up: Turtle movements don't draw lines.
echo           penDown  (pd)   Put pen down: Turtle movements draw lines again.
echo/
echo color     setColor (sc)   Set the color of the drawing (in 0..15 range).
echo color     setBack  (sb)   Set the background color of the window (in 0..15).
echo/
echo           home            Moves Turtle to center of window, heading upwards.
echo           clearScreen(cs) Clears the graphics window and reset all parameters.
echo/
echo Some simple examples:
echo    100 forward   45. right   13 setColor   50 forward
echo    90. left  5 ( 100 forward  72. right  --? : ) 
echo    12 ( 190 forward  150. right  --? : )
echo    0 ( ^> forward  30. right  ++ 100 ^<=x? ^< : )
echo/
echo -^> Remember that number of degrees for rotations *MUST HAVE* decimal point ^<-
echo/
echo To end the drawing, enter: end
%break%
echo For help, enter: help
set "Msg[Title]=Enter new window title (or help): "
set "Msg[Line]=Select line thickness "
set "Msg[Window]=Enter X,Y window sizes in pixels [^!sizes^!]: "
set "Msg[Window2]=-^^> It is suggested to move "^^!title^^!" window to a visible area ^^<-"
set "Msg[End]=To close the window, select it and press any key"
set "Msg[Ready]=Ready..."
:ENDIF

set "sizes[1]=400,400"
set "sizes[2]=200,200"
set "sizes[3]=150,150"

:nextWindow
echo/
echo ================================
echo/
set "title="
set /P "title=%Msg[Title]%"
if not defined title goto begin
if /I "%title%" == "help" goto help
if /I "%title%" == "ayuda" goto helpLA
choice /C 123 /M "%Msg[Line]%"
set "lineSize=%errorlevel%"
set "sizes=!sizes[%lineSize%]!"
set /P "sizes=%Msg[Window]%"
for /F "tokens=1,2 delims=,-: " %%x in ("%sizes%") do set /A "MaxX=%%x, MaxY=%%y, HomeX=MaxX/2, HomeY=MaxY/2"
(
echo %HomeX% %HomeY% 15 0 %MaxX% %MaxY%
echo T 90.0
) > PrevData.txt
rem/> PipeFile.txt
start "* %title%" cmd /K "%~F0" DrawInteractive ^< PipeFile.txt  2^> ErrorFile.txt
waitFor StartDraw > NUL
call "%~F0" GetCommands 3>> PipeFile.txt  4< ErrorFile.txt
goto nextWindow


:GetCommands
if defined Msg[Window2] (
   echo/
   echo %Msg[Window2]%
   set "Msg[Window2]="
)
set "end=/end/fin/"

CHCP 850 > NUL

:NextCommand
   echo/
   echo %Msg[Ready]%
   set "command="
   set /P "command=> "
   if not defined command goto NextCommand
   if /I "!command!" == "help" set "break=goto GetCommands" & goto help2
   if /I "!command!" == "ayuda" set "break=goto GetCommands" & goto help2LA
   (
   set /P "=!command!" < NUL
   echo/
   ) >&3
   waitFor EndDraw > NUL
   set "error="
   set /P "error=" <&4
   if defined error echo ERROR: %error%
if "!end:/%command:~0,3%/=!" == "%end%" goto NextCommand

CHCP 65001 > NUL

echo/
echo %Msg[End]%
exit /B


:DrawInteractive

title * %title%
PIXELFNT %lineSize%
mode %MaxX%,%MaxY%
set "color=0123456789ABCDEF"
set "clear=/clearScreen/cs/borraPantalla/bp/"
set "end=/end/fin/"
call :printf "CURS9 st"
CScript //nologo //E:JScript "%~F0"
waitFor /SI StartDraw > NUL

:newCommand
set "init=ht"
set /A "next=1%time:~6,2%*100+1%time:~9,2%-100 + 53"

:getCommand
set "command="
set /P "command="
if defined command goto drawCommand
set /A "now=1%time:~6,2%*100+1%time:~9,2%-100"
if "%time:~6,2%" equ "00" set /A "next=now + 26"
if %now% lss %next% goto getCommand

:blink
call :printf ht
if "%init%" == "" (set "init=ht") else (set "init=")
set /A "next=1%time:~6,2%*100+1%time:~9,2%-100 + 53"
goto getCommand

:drawCommand
for /F %%a in ("!command!") do (
   if "!end:/%%a/=!" neq "%end%" goto CloseInteractive
   if "!clear:/%%a/=!" neq "%clear%" set "init="
)

set "command=%init% !command! st"
call :printf
if not errorlevel 0 call :printf ht
waitFor /SI EndDraw > NUL
CScript //nologo //E:JScript "%~F0"
goto newCommand

:CloseInteractive
title %title%
if "%init%" neq "" call :printf ht
waitFor /SI EndDraw > NUL
echo @echo off ^& pause ^> NUL ^& exit  > END.bat
END < CON 2> CON

=======================================

Most of the figures below were taken from a Logo contest at this site:
http://www.mathcats.com/gallery/15wordcontest.html

"Five Roses" figure is a special case. The fraction stored at FP1 storage register define the number of flowers:
0.1 = 5 flowers, 0.2 = 10 flowers, 0.125 = 8 flowers and 0.15 = 20 flowers. The number of "steps" is: flowers * 360
More details at: http://www.mathcats.com/gallery/fiverosedetails.html

:Option_2	Draw predefined figures

rem Define the predefined figures
set "figures=0"
set "options="
set "letter=_ABCDEFGHIJKLMNOPQRSTUVWXYZ"

rem Question mark "?" changed by "Q" in the formulae below to avoid problems with FOR command:

for %%a in (
   "Triangles=3 ( 200 fd  2 ( 120. rt  100 fd  --Q : ) <  2 ( 120. lt  100 fd  --Q : ) <  120. rt --Q : ) <"
   "Star-6=6 ( 4 ( 100 fd 120. lt --Q : ) < 180. lt --Q : ) <"
   "Triade=3 ( 2 ( 30 ( 8 fd RCL.0 rt --Q : ) < RCL.0 CHS STO.0 DROP --Q : ) < 120. lt --Q : ) <"
   "Pentahexagon=5 ( 6 ( 100 fd 72. rt --Q : < ) 144. rt --Q : )"
   "Hypercube=8 ( 4 ( 90. rt 100 fd --Q : ) < 100 bk 45. lt --Q : ) <"
   "Hexagon=100 ( 6 ( 6 ( >3 fd 60. lt --Q : ) < 60. lt --Q : ) < 50 - Q : ) <"
   "Hexagon Variation= 90 ( 6 ( 6 ( >3 fd 60. rt --Q : ) < 60. rt --Q : ) < 15 - Q : ) <"
   "Penta-Rep=18. rt  5 ( 72. rt 200 ( 5 ( >2 fd 144. rt --Q : ) < 20 - Q : ) < --Q : ) <"
   "Octa-Star Spiral=120. lt  4 ( 8 ( >2 fd 135. rt --Q : ) < > fd 30. rt  4 + 100 >=Q < : < ) <"
   "Penta-Star Spiral= 60. rt  6 ( 5 ( >2 fd 144. rt --Q : ) < > fd 30. rt  3 +  96 >=Q < : < ) <"
   "Spinning=24 ( 25 ( 35 fd 35 bk 15. rt --Q : ) < 35 fd --Q : ) <"
   "Rotating Circle=36 ( 34 ( 12 fd  10. rt --Q : ) < 90. rt --Q : ) <"
   "Circles=number ( angle. rt  60 ( 6. rt 8 fd --Q : ) < --Q : ) <"
   "Roses=steps ( 10 fd > . RCL1 ADD rt --Q : ) < 10 fd"
   "Dahlia=8 ( 45. rt  param (  90 ( 2 fd  2. rt --Q : ) <  90. rt --Q : ) <  --Q : ) <"
   ) do for /F "tokens=1,* delims==" %%x in (%%a) do (
      set /A figures+=1
      set "option[!figures!]=%%x"
      set "code[!figures!]=%%y"
      for %%i in (!figures!) do set "options=!options!!letter:~%%i,1!"
   )
)

cls
echo/
=IF= /I "%RegionCode%" NEQ "LA" GOTO :ELSE
echo          Gráficas de la Tortuga en printf.exe, por Antonio Pérez Ayala
if defined openSite goto continue
   echo/
   echo La mayoría de las figuras predefinidas que se muestran a continuación
   echo fueron tomadas de un sitio de Logo (en inglés) y traducidas a printf.exe.
   echo/
   choice /M "¿Desea abrir el sitio de las figuras de Logo "
   if %errorlevel% equ 1 explorer "http://www.mathcats.com/gallery/15wordcontest.html" & set "openSite=T"
:continue
set "Msg[Figures]=Estas son las figuras predefinidas:" 
set "Msg[Figures2]=-^> Después que cada figura se trace, oprima una tecla para regresar aquí ^<-"
set "Msg[Figure]=Seleccione la figura deseada (Y=lista, Z=termina): "
set "Msg[Circles]=Ingrese el número de círculos (factor de 360) [20]: "
set "Msg[BadCircles]=ERROR: Debe ser un submúltiplo de 360"
set "Msg[Roses]=Ingrese un número entre 3 y 20 [10]: "
set "Msg[BadRoses]=ERROR: Número inválido"
set "Msg[NoRoses]=Este número no tiene una figura definida"
set "Msg[Dahlia]=Ingrese el valor del parámetro entre 1 y 7: "
=GOTO= :ENDIF
:ELSE
echo           Turtle Graphics in printf.exe, by Antonio Perez Ayala
if defined openSite goto continue
   echo/
   echo Many of the predefined figures that will be displayed below were taken
   echo from a Logo contest site and translated to printf.exe RPN expressions.
   echo/
   choice /M "Do you want to open the Logo site "
   if %errorlevel% equ 1 explorer "http://www.mathcats.com/gallery/15wordcontest.html" & set "openSite=T"
:continue
set "Msg[Figures]=These are the predefined figures:"
set "Msg[Figures2]=-^> After each figure is drawn, press a key to return here ^<-"
set "Msg[Figure]=Select the desired figure (Y=list, Z=end): "
set "Msg[Circles]=Enter number of circles (factor of 360) [20]: "
set "Msg[BadCircles]=ERROR: Must be a submultiple of 360"
set "Msg[Roses]=Enter a number in 3..20 range [10]: "
set "Msg[BadRoses]=ERROR: Invalid number"
set "Msg[NoRoses]=This number have not a defined figure"
set "Msg[Dahlia]=Enter value of parameter in 1..7 range: "
:ENDIF

set "endShowFigures="
:showFigures
echo/
echo %Msg[Figures]%
echo/
for /L %%i in (1,1,%figures%) do echo    !letter:~%%i,1!. !option[%%i]!
%endShowFigures%
echo/
echo %Msg[Figures2]%
if /I "%RegionCode%" equ "LA" (
   echo/
   echo Los comandos equivalentes en español del código original en inglés son:
   echo     bk-^>re, fd-^>av, lt-^>iz, pd-^>bl, pu-^>sl, rt-^>de, sb-^>pf, sc-^>pc
)

:nextFigure
echo/
set /P "=%Msg[Figure]%" < NUL
choice /C YZ%options% /N > NUL
set /A "i=%errorlevel%-2"
if %i% == -1 echo/& set "endShowFigures=goto nextFigure" & goto showFigures
if %i% == 0 goto begin
set "title=!option[%i%]!"
set "command=!code[%i%]:Q=?!"
echo !letter:~%i%,1!. %title% code:
echo !command!
set "init="
if "%title%" equ "Triangles" set "init=[5 8 / _ [4 4 / _ pu setPos pd"
if "%title%" equ "Star-6" set "init=[5 4 / _ [4 8 / pu setPos pd 90. lt"
if "%title%" equ "Triade" set "init=pu 160 bk pd 60. lt 6. STO.0"
if "%title%" equ "Pentahexagon" set "init=0 50 pu setPos pd  90. lt"
if "%title%" equ "Hypercube" set "init=[5 8 /   [4 4 / _ pu setPos pd"
if "%title%" neq "%title:Spiral=%" set "init=0  [4 12 / _ pu setPos pd"
if "%title%" equ "Spinning" set "init=0 [4 3 / _ pu setPos pd"
if "%title%" neq "Circles" goto endif
   :getNumber
   set "number=20"
   set /P "number=%Msg[Circles]%"
   set /A "mod=360 %% number, angle=360 / number"
   if %mod% neq 0 echo %Msg[BadCircles]% & goto getNumber
   set "title=%title% %number%"
   set "command=!command:number=%number%!"
   set "command=!command:angle=%angle%!"
:endif
if "%title%" neq "Roses" goto endif
   :getDivisor
      set "divisor=10"
      set /P "divisor=%Msg[Roses]%"
      set /A "divisor=divisor" 2> NUL
      if %divisor% geq 3 if %divisor% leq 20 goto continue
      echo %Msg[BadRoses]%
   goto getDivisor
   :continue
   for /F "tokens=%divisor%" %%n in ("_ _ 6 4 10 0 14 8 18 5 23 12 26 0 30 16 34 0 38 20") do set /A "turns=%%n, steps=turns*360"
   if %turns% equ 0 echo %Msg[NoRoses]% & goto getDivisor
   set "title=%title% #%divisor% = %turns% flowers"
   if %divisor% equ 3 set "command=!command:10=6!"
   if %divisor% equ 4 set "command=!command:10=8!"
   set "command= %divisor% > . INV }3 ( 10 ==? 0.001 ; ZERO ) <3 <3 ADD STO1 DROP  !command:steps=%steps%!"
   set "init=[5 10 / [4 10 / pu setPos pd"
:endif
if "%title%" neq "Dahlia" goto endif
   choice /N /C 1234567 /M "%Msg[Dahlia]%"
   set "param=%errorlevel%"
   set "title=%title% %param%"
   set "command=!command:param=%param%!"
   set /A "mod4=param %% 4"
   if %mod4% neq 0 set "init=[5 10 / _ [4 4 / _ pu setPos pd"
:endif
set /A "MaxX=400, MaxY=400, HomeX=MaxX/2, HomeY=MaxY/2"
(
echo %HomeX% %HomeY% 15 0 %MaxX% %MaxY%
echo T 90.0
) > PrevData.txt
set "command=%init% !command!"
start "" cmd /K "%~F0" DrawFigure
goto nextFigure


:DrawFigure

title !letter:~%i%,1!. %title%
PIXELFNT 1
mode %MaxX%,%MaxY%
call :printf
pause > NUL
CScript //nologo //E:JScript "%~F0"
echo @echo off ^& pause ^> NUL ^& exit  > END.bat
END


Turtle Graphics runtime values are kept in these registers:

I0	X position
I1	Y position
I2	Line Color
I3	Back Color
I4	MaxX window
I5	MaxY window
I6	Font Width
I7	Font Height
I8	MultX Font factor
I9	MultY Font factor

FP0	Angle. (Turtle heading)

F0	Pen up/down

:printf
if "%~1" neq "" set "command=%~1"
printf				/* Turtle Graphics support procedures:	*/ ^
   forward( fd )		/* pixels  Forward:			*/ ^
   fd( fdMov Line )		/* - draw a line for forward movement	*/ ^
   fx( fdMov XLine )		/* - draw a line with XOR colors	*/ ^
   back( bk )			/* pixels  Back:			*/ ^
   bk( _ fdMov Line )		/* - draw a line for backward movement	*/ ^
   right( rt ) rt( STO-0 DROP )	/* Right Turn				*/ ^
   left( lt )  lt( STO+0 DROP )	/* Left Turn				*/ ^
   penUp( pu )    pu( CF0 )	/* Pen Up				*/ ^
   penDown( pd )  pd( SF0 )	/* Pen Down				*/ ^
   setColor( sc )		/* Set foreground Color:		*/ ^
   sc( 15 ^& ]2 ^< )		/* - just store it			*/ ^
   setBack( sb )		/* Set Background color:		*/ ^
   sb( 15 ^& ]3			/* - store color and activate it:	*/ ^
      "0123456789ABCDEF"	/* B# "0123456789ABCDEF"		*/ ^
      ^<^> getc0 ^<2		/* "0123456789ABCDEF" 'B'		*/ ^
      ^<^> [2 getc0 ^<2 ^<2	/* 'B' 'F'				*/ ^
      "COLOR bf" 7 }3 putc0	/* 'B' "COLOR bF" 7			*/ ^
      -- }3 putc0 ^<		/* "COLOR BF"				*/ ^
      SYSTEM ^< )		/* Set Background color on the screen	*/ ^
   home( [5 2 / [1 - [4 2 / [0 - Line	/* line to Home position	*/ ^
         90.0 STO0 DROP )	/* - and heading Turtle upwards		*/ ^
   clearScreen( cs )		/* Clear Screen:			*/ ^
   cs( "COLOR & CLS" SYSTEM ^<	/* - clear the screen,			*/ ^
       [5 2 / ]1 ^< [4 2 / ]0 ^< CURS2	/* - go Turtle home		*/ ^
       90.0 STO0 DROP		/* - heading Turtle upwards		*/ ^
       15 ]2 ^< 0 ]3 ^< pd )	/* - and reset colors and pen down	*/ ^
   setPos( sp )			/* Set Turtle Position:			*/ ^
   sp( [4 2 / + [-0 ^<^>	/* - get movement to new position	*/ ^
       [5 2 / ^<^> - [-1 ^<^> Line )	/* - and Line to such position	*/ ^
   setHead( sh )		/* Set Turtle Heading:			*/ ^
   sh( 90. XCHG SUB STO0 DROP )	/*					*/ ^
   st( ht )			/* Show/Hide Turtle (using XOR line)	*/ ^
   ht( CURS			/* get position				*/ ^
       90. rt 7 fx 135. lt 10 fx 90. lt 10 fx 135. lt 7 fx 90. lt /* turtle */ ^
       ]0 ^< ]1 ^< CURS2 )	/* restore position			*/ ^
				/* ======= Spanish Procedures  =======	*/ ^
   avanza( fd )    av( fd )	/* avanza           = forward		*/ ^
   retrocede( bk ) re( bk )	/* retrocede        = backward		*/ ^
   derecha( rt )   de( rt )	/* derecha          = right		*/ ^
   izquierda( lt ) iz( lt )	/* izquierda        = left		*/ ^
   subeLapiz( pu ) sl( pu )	/* subeLápiz        = penUp		*/ ^
   bajaLapiz( pd ) bl( pd )	/* bajaLápiz        = penDown		*/ ^
   ponColor( sc )  pc( sc )	/* ponColor         = setColor		*/ ^
   ponFondo( sb )  pf( sb )	/* ponFondo         = setBackground	*/ ^
   centro( home )		/* centro (center) ~= home		*/ ^
   borraPantalla( cs ) bp( cs ) /* borraPantalla    = clearScreen	*/ ^
   ponPos( sp )    pp( sp )	/* ponPosición      = setPosition	*/ ^
   ponRumbo( sh )  pr( sh )	/* ponRumbo        ~= setHeading	*/ ^
   "No format"			/* ===== Main program start here =====	*/ ^
				/* Recover values from previous run:	*/ ^
   "PrevData.txt" IN{?:0	/* Open PrevData.txt file for input	*/ ^
   80 IN?:0 ^<			/* "X Y Color Back MaxX MaxY"		*/ ^
   atoi1			/* X Y Color Back MaxX MaxY 6		*/ ^
/" < ]5 < ]4 < ]3 < ]2 < ]1 < ]0 < /"	/* Initialize Integer values	*/ ^
   80 IN?:0 ^<			/* "PenDown Angle"			*/ ^
   0 getc			/* "PenDown Angle" 0 'p'		*/ ^
   CF0 ( 'T' ==? SF0 )		/* Set Pen status			*/ ^
   ^< ^< ^< 2 +			/* "Angle"				*/ ^
   atof STO0 DROP		/* Store in its place			*/ ^
   80 ( IN?:0 )			/* Close input handle 0			*/ ^
   CURS2			/* Move cursor to last position		*/ ^
   !command!			/* Execute the given command		*/ ^
				/* Save values for next run:		*/ ^
   "PrevData.txt" OUT{:1		/* Create data file for output	*/ ^
   FMT{ "%%i %%i %%i %%i %%i %%i\n"	/* Out integer values		*/ ^
        [0 [1 [2 [3 [4 [5 OUT:1 FMT}	/*				*/ ^
   FMT{ "%%c %%.4f\n"			/* Out other values		*/ ^
       ( F0? 'T' ; 'F' ) RCL0 OUT:1 FMT} /*				*/ ^
   OUT}:1			/* Close PrevData.txt file		*/

exit /B


:Option_3	Terminate
cls
goto :EOF


@end

// Reactivate original command window after a graphic is drawn
new ActiveXObject("WScript.Shell").AppActivate("Turtle Graphics")
