@if (@CodeSection == @Batch) @then

@echo off
setlocal DisableDelayedExpansion
if "%~1" neq "" if /I "%~X1" neq ".fdf" goto %1
goto begin

Turtle Graphics runtime values are kept in these registers:

I0	X position
I1	Y position
I2	Draw Color
I3	Back Color
I4	MaxX window
I5	MaxY window
I6	Font Width
I7	Font Height
I8	MultX Font factor
I9	MultY Font factor

FP0	Angle. (Turtle heading)

F0	Pen up/down


Text output facility via Turtle Graphics forming the shape of each character.
Designed and programmed by Antonio Perez Ayala, July/2024

This feature is comprised of two procedures: SetFont that read a Font Definition File,
and Label that output a text message using such definitions. Label uses Poly predefined
function that draws a series of "poly points" stored in storage registers.

The integer storage registers are used to store the font definitions in this way:

R32..R255: These registers corresponds to the ASCII code of each character and points
           to the start of its equivalent definition in following storage registers.

The character definitions starts at R256 and are comprised of Y,X couples.
The end of each character definition is marked by a value greater than 4000.
This end value is not in the font file; it is inserted by SetFont procedure.

For example, if the "A" character definition is comprised of 19 Y,X pairs (38 numbers),
and "B" character definition is comprised of 27 Y,X pairs (54 numbers), then:

'A' = 65, R65 = 256, R256..R293 = definition of 'A' + R294=5000. Next start at R295
'B' = 66, R66 = 295, R295..R348 = definition of 'B' + R349=5000. Next start at R350
'C' = 67, R67 = 350, etc...

SetFont procedure read a Font Definition File and populate the integer storage registers
as described before. It also initializes registers R6..R9. The file name is eliminated.

    "FontDefinitionFile.fdf" SetFont

The Font Definition File contains Y,X couples for the *relative movement* of each of the segments
that comprise a character, starting at the base line where the characters will be displayed.
A 0,0 pair indicate to jump to the next movement with no drawing. For example:

A 10,0 2,2 0,3 -2,2 -10,0 0,-2 5,0 0,-3 -5,0  0,0,7,2  2,0 1,1 0,1 -1,1 -2,0 0,-3  0,0,-7,8
B 12,0 0,5 -2,2 -3,0 -1,-1 -1,1 -3,0 -2,-2 0,-5  0,0,2,2  3,0 0,2 -1,1 -1,0 -1,-1 0,-2  0,0,5,0  3,0 0,2 -1,1 -1,0 -1,-1 0,-2  0,0,-7,8

A value of 1000 indicate to set the "pixel" of the current position with no movement.
This is useful to complete the end of lines that does not form a closed figure,
particularly in small characters. See examples in the included font files.

This series of values are the ones that uses Poly procedure to draw the figure.
It also uses registers R8=XMul and R9=YMul as scale size factors for the font. 

Label procedure display the string given as parameter using Poly on each character.
The string parameter is eliminated.

    "Message" Label

A couple examples of Font Definition Files called "APA Big Font 7x12.fdf" and "APA Small Font 3x6.fdf"
are included in printf.exe package. You can create your own Font files in a very simple way using a
standard text editor, like Windows Notepad. Be aware that the file *must* be saved as ANSI encoding (no UTF).


:begin

CHCP 65001 > NUL

=IF= /I "%RegionCode%" NEQ "LA" GOTO :ELSE
set "Msg[NoFont]=ERROR: No existe el archivo del Font"
set "Msg[Title]=Ingrese el Título de una nueva ventana para trazar rótulos: "
set "Msg[startHere]=comience aquí"
set "Msg[Window]=Ingrese el tamaño (X,Y) de la Ventana en pixels"
set "Msg[invalid]=Error: tamaño inválido"
set "Msg[Back1]=Para cambiar el color del fondo al color actual de letras"
set "Msg[Back2]=debe cambiar primero el color de letras (no pueden ser iguales)"
set "Msg[Colors]=Colores disponibles:"
set "Msg[Fonts]=Tipos de letra disponibles:"
set "Msg[GetOne]=Seleccione uno"
set "Msg[Font]=Ingrese el tamaño (X,Y) de letra (factores de multiplicación)"
set "Msg[Labels1]=-^> Después que el texto se trace, oprima una tecla para regresar aquí ^<-"
set "Msg[Labels2]=Ingrese el texto de los rótulos:"
set "Msg[Ready]=Listo"
set "Msg[End1]=Ventana "!title!" completada"
set "Msg[End2]=Para cerrar la ventana, selecciónela y oprima una tecla"
=GOTO= :ENDIF
:ELSE
set "Msg[NoFont]=ERROR: Font file not found"
set "Msg[Title]=Enter Title for a new window to draw labels: "
set "Msg[startHere]=start here"
set "Msg[Window]=Enter size (X,Y) of the window in pixels"
set "Msg[invalid]=Error: invalid size"
set "Msg[Back1]=In order to change background color to current font color"
set "Msg[Back2]=you must change the font color first (can not be the same)"
set "Msg[Colors]=Available Colors:"
set "Msg[Fonts]=Available Fonts:"
set "Msg[GetOne]=Select the desired one"
set "Msg[Font]=Enter size (X,Y) of the font (multiplier factors)"
set "Msg[Labels1]=-^> After the text was drawn, press a key to return here ^<-"
set "Msg[Labels2]=Enter labels text:"
set "Msg[Ready]=Ready"
set "Msg[End1]="!title!" window completed"
set "Msg[End2]=To close the window, select it and press any key"
:ENDIF

title Main Turtle Graphics Labels
cls
echo/
echo Turtle Graphics Labels

set "myself=%~F0"
cd "%~P0"
set "font="
if /I "%~X1" equ ".fdf" set "font=%~NX1"
if not defined font for %%f in (*.fdf) do set "font=%%f" & goto continue
:continue
if not exist "%font%" echo %Msg[NoFont]% & pause & goto :EOF
set /P "line=" < "%font%"
for /F "tokens=2,3 delims=, " %%a in ("%line%") do set /A "height=%%a, width=%%b"
set /A "mulX=5,mulY=5, sizeX=width*mulX*17, skip=height*mulY+6, sizeY=(skip+4)*6, color=15, backColor=0"

set "title=Turtle Graphics Labels - "%font%" @ %mulX%x%mulY% - Standard ASCII characters 32-127"
start "" cmd /K ^""%~F0" DrawLabels " !#$%%%%&'()*+,-./" "0123456789:;<=>?" "@ABCDEFGHIJKLMNO" "PQRSTUVWXYZ[\\]^_" "`abcdefghijklmno" "pqrstuvwxyz{|}~"^"
waitFor EndDraw > NUL

set "title=Turtle Graphics Labels - "%font%" @ %mulX%x%mulY% - Extended characters 128-255 of Code Page 850"
start "" cmd /K ^""%~F0" DrawLabels "ÇüéâäàåçêëèïîìÄÅ" "ÉæÆôöòûùÿÖÜø£Ø×ƒ" "áíóúñÑªº¿®¬½¼¡«»" "ÁÂÀ©¢¥ãÃ¤ðÐÊËÈÍÎÏÌ" "ÓßÔÒõÕµþÞÚÛÙýÝ¯´" "­±¾¶§÷¸°¨·¹³² "^"
waitFor EndDraw > NUL

goto nextWindow


:DrawLabels

title %title%
PIXELFNT 1
mode %sizeX%,%sizeY%

:loop
   shift
   if "%~1" equ "" goto endLoop
   call :Draw %1
goto loop
:endLoop

CScript //nologo //E:JScript "%myself%" "Main"
waitFor /SI EndDraw > NUL
echo @echo off ^& pause ^> NUL ^& exit  > END.bat
END


:nextWindow
echo/
echo ======================
echo/
set "title="
set /P "title=%Msg[Title]%"
echo/
if not defined title cls & goto :EOF

set "startHere=^<- %Msg[startHere]%"
rem/> PipeFile.txt
start "" cmd /K "%~F0" DrawInteractive ^< PipeFile.txt
waitFor StartDraw > NUL
call "%~F0" GetCommands 3>> PipeFile.txt
goto nextWindow


:GetCommands
setlocal EnableDelayedExpansion

cls
echo/
set "colorsHex= 0123456789ABCDEF"
set /A "aux=color+1"
set "colorC=!colorsHex:~%aux%,1!"

=IF= /I "%RegionCode%" NEQ "LA" GOTO :ELSE
echo Usted está trazando rótulos en la ventana "%title%"
echo/
echo ¿Qué desea hacer?
echo/
echo    1. Cambiar el tamaño de la Ventana (borra todo, tamaño actual: %sizeX%,%sizeY%)
echo    2. Cambiar el color del Fondo (pierde colores de los rótulos, actual: %backColor%)
echo    3. Cambiar el Tipo de letra (actual: "%font%")
echo    4. Cambiar el tamaño de Letra (actual: %mulX%,%mulY%)
echo    5. Cambiar el Color de letras (actual: %color%)
echo    6. Ingresar y mostrar algunos rótulos      %startHere% & set "startHere="
echo    7. Borrar todos los rótulos
echo    8. Terminar el trazo y regresar
echo/
choice /N /C "12345678" /M "Seleccione la acción deseada: "
echo/
goto Action-%errorlevel%
:ELSE
echo You are drawing labels on "%title%" window
echo/
echo What do you want to do?
echo/
echo    1. Change window Size (clear all, current size: %sizeX%,%sizeY%)
echo    2. Change Background color (loss all labels colors, current: %backColor%)
echo    3. Change the Font (current: "%font%")
echo    4. Change font Size (current: %mulX%,%mulY%)
echo    5. Change font Color (current: %color%)
echo    6. Enter and show some Labels           %startHere% & set "startHere="
echo    7. Clear all labels
echo    8. End drawing and go back
echo/
choice /N /C "12345678" /M "Select desired action: "
echo/
goto Action-%errorlevel%
:ENDIF

:Action-1	Set window Size
set "sizes0=%sizeX%,%sizeY%"
:getWinSize
set "sizes=%sizes0%"
set /P "sizes=%Msg[Window]% [%sizes%]: "
for /F "tokens=1,2 delims=,- " %%a in ("%sizes%") do set /A "mul=0,sizeX=%%a,sizeY=%%b,mul=sizeX*sizeY" 2> NUL
if not defined sizeX goto error
if not defined sizeY goto error
if "%mul%" neq "0" goto continue
:error
echo %Msg[invalid]% & goto getWinSize
:continue
echo mode %sizeX%,%sizeY% >&3
goto actionEnd

:Action-2	Set Background color
echo %Msg[Back1]%
echo %Msg[Back2]%
echo/
echo %Msg[Colors]%
echo/
color /? | findstr /RC:"^  *[0-9]"
echo/
choice /N /C %colorsHex% /M "%Msg[GetOne]% [%backColor%]: "
echo color !colorsHex:~%errorlevel%,1!%colorC% >&3
set /A "backColor=%errorlevel%-1"
goto actionEnd

:Action-3	Set Font
set "options="
set "i=0"
echo %Msg[Fonts]%
echo/
for %%a in (*.fdf) do (
   set /A i+=1
   echo    !i!. %%~Na
   set "font[!i!]=%%a"
   set "options=!options!!i!"
)
echo/
choice /N /C %options% /M "%Msg[GetOne]%: "
set "font=!font[%errorlevel%]!"
set /P "line=" < "%font%"
for /F "tokens=2,3 delims=, " %%a in ("%line%") do set /A "height=%%a, skip=height*mulY+2, width=%%b"
echo set "font=%font%" >&3
echo set "skip=%skip%" >&3
goto actionEnd

:Action-4	Set font Size
set "sizes0=%mulX%,%mulY%"
:getFontSize
set "sizes=%sizes0%"
set /P "sizes=%Msg[Font]% [%sizes%]: "
for /F "tokens=1,2 delims=,- " %%a in ("%sizes%") do set /A "mul=0,mulX=%%a,mulY=%%b,mul=mulX*mulY, skip=height*mulY+2" 2> NUL
if not defined mulX goto error
if not defined mulY goto error
if "%mul%" neq "0" goto continue
:error
echo %Msg[invalid]% & goto getFontSize
:continue
echo set /A "mulX=%mulX%,mulY=%mulY%,skip=%skip%" >&3
goto actionEnd

:Action-5	Set label Color
echo %Msg[Colors]%
echo/
color /? | findstr /RC:"^  *[0-9]"
echo/
choice /N /C %colorsHex% /M "%Msg[GetOne]% [%color%]: "
set /A "color=%errorlevel%-1" & set "colorC=!colorsHex:~%errorlevel%,1!"
echo set "color=%color%"  >&3
goto actionEnd

:Action-6	Enter and show Labels
echo %Msg[Labels1]%
echo/
echo %Msg[Labels2]%
echo/
rem Next line required to show extended characters correctly
CHCP 850 > NUL
:nextLabel
   set "label="
   set /P "label=> "
   if not defined label goto endLabels
   echo call :Draw "!label!" >&3
goto nextLabel
:endLabels
CHCP 65001 > NUL
goto actionEnd

:Action-7	Clear all labels
echo cls >&3
:actionEnd
echo %Msg[Ready]%
pause
goto GetCommands

:Action-8	End Labels
>&3 echo end
echo %Msg[End1]%
echo/
echo %Msg[End2]%
goto :EOF


:DrawInteractive
setlocal EnableDelayedExpansion

title * %title%
PIXELFNT 1
mode %sizeX%,%sizeY%
CScript //nologo //E:JScript "%myself%" "Main"
waitFor /SI StartDraw > NUL

:getCommand
   set "command="
   set /P "command="
   if not defined command goto getCommand
   if "!command!" equ "end" goto endCommands
   if "!command:~0,10!" equ "call :Draw" CScript //nologo //E:JScript "%myself%" "%title%"
   !command! 2>NUL
   if "!command:~0,10!" equ "call :Draw" pause < CON > NUL & CScript //nologo //E:JScript "%myself%" "Main"
goto getCommand

:endCommands
title %title%
waitFor /SI EndDraw > NUL
echo @echo off ^& pause ^> NUL ^& exit  > END.bat
END < CON 


Next routine is called with each label (line). Note that Draw routine do a
complete initialization of all Turtle Graphics parameters, including load the font
file. This is inefficient. A better option would be to load the font just once
and then read the lines in the printf.exe code itself, but doing that will make
the RPN code more complex and add elements that are not of general use.

To add Label capability to your own programs just include on they the code of
SetFont, SetSizes and Label.

:Draw
setlocal DisableDelayedExpansion

set "param=%~1"
set "param=%param:^^=^%"
for /L %%i in (1,1,%skip%) do echo/

printf   ^
   SetFont(			/*			  PROC SetFont: Load a Font Definition File	*/	^
      IN{?:1			/* 			     IF Open font file				*/	^
      256 ]9			/* 				Initialize first position: R9=256	*/	^
      (  ^<*			/*				Clear stack and string data		*/	^
         1000 IN?:1		/* "65 10,0 2,2  -7,8" len	WHILE NOT EOF: Read next line		*/	^
         (  ?  ^< 		/* "65 10,0 2,2  -7,8"		  IF Line not empty			*/	^
            "//" 2 cmps2	/* "65 10,0 2,2  -7,8" "//" 2 R     Compare line vs. "//" and get Result*/	^
            (  ? ^< ^< ^<	/* "65 10,0 2,2  -7,8"		    IF Line not start in "//"		*/	^
               atoi1		/* 65 10 0 2 2  -7 8 39		      Convert numbers and count them	*/	^
               shift1		/* 10 0 2 2  -7 8 38 65		      Get first number 65 = 'A'		*/	^
               ]I ^< [9 ]i ]I ^< /*10 0 2 2  -7 8 38		      RI=65, R65=256, RI=256		*/	^
               (  shift1	/* 0 2 2  7 8 37 10		      DO while get next Y number	*/	^
                  ]i++		/* 0 2 2  7 8 37 10			Store: R256=10 and advance I=257*/	^
                  (  1000 !=?	/* 0 2 2  7 8 37 10 1000		IF the number is not 1000	*/	^
                     ^< ^<	/* 0 2 2  7 8 37			  Drop 1000 and number		*/	^
                     shift1	/* 2 2  7 8 36 0			  Get next X number		*/	^
                     ]i++ ^< ;	/* 2 2  7 8 36				  Store: R257=0 and advance i=258 */	^
                    ^< ^<	/* 0 2 2  7 8 37			ELSE just drop 1000 and number	*/	^
                  )		/*					ENDIF				*/	^
                  ? :		/* 2 2  7 8 36			        More numbers? go back	   	*/	^
               )		/*				      WHILE get next Y number		*/	^
				/* 0		After last number:    Stored: R293=8 and advance I=294	*/	^
               5000 ]i++	/* 0 5000			      R294=5000 and advance I=295	*/	^
               [I ]9 ^<		/* 0 5000 295			      Last pos for next char: R9=295	*/	^
            )			/*				    ENDIF				*/	^
         )			/*				  ENDIF					*/	^
         :			/*				  Go back to read next line		*/	^
      )				/*				ENDWHILE				*/	^
      127 ]I ^< [i		/* char 127			Get pointer to ASCII char 127 (Del)	*/	^
      ( ? ; ^< 256 )		/* char 127 or first defined	If not defined: change it by first char	*/	^
      32 ]I ^< 224 ]9 ^<	/* some char			RI=32, R9=224				*/	^
      ( [i ( ? ^< ; ^< ]i ) ]++I ]--9? : ) ^<	/*		Initialize not defined characters	*/	^
      5 ]8 _ ]9 ^<		/*				Initialize font size scale factors = 5	*/	^
   ;				/*			     ELSE:					*/	^
      ^< FMT{ "Font Definition File not found" OUT FMT}		/* Font file not found			*/	^
   )				/*			  ENDPROC SetFont				*/	^
				/*									*/	^
   SetSizes( ]8 ^< _ ]9 ^< )	/*			  PROC SetSizes: store character sizes for font	*/	^
				/*									*/	^
   PolX(			/* 'C'			  PROC Poly: draw R['C']-^> polyPoints		*/	^
      CF1			/* 			     clear 0,0 flag				*/	^
      ]I ^< [i ]I ^<		/* RI='C', RI=R['C']	     initialize I = R['C']			*/	^
      [i++			/* Y1			     recall Y1 and advance RI			*/	^
      (				/*			     DO (while Y value LSS 4000)		*/	^
         [*9			/* Y1*Ymul			scale Y1				*/	^
         [i++ [*8		/* Ymov X1*XMul			recall X1 and advance RI, scale X1	*/	^
         ^>2 ^>2		/* Ymov Xmov Ymov Xmov		duplicate both Ymov and Xmov		*/	^
         (  ^|?			/* Ymov Xmov OR			IF both numbers are not zero		*/	^
            ^< Line		/*				   drop OR, draw line (or just jump)	*/	^
            SF0			/*				   reset "no draw" flag			*/	^
            ;			/*				ELSE					*/	^
            ^< ^< ^< CF0	/*				   drop OR and movs, set "no draw"	*/	^
         )			/*				ENDIF					*/	^
         [i++			/* Y2,  Y3,  ...		recall next Y and advance I		*/	^
         ( 1000 ==?		/*				IF Yn value is 1000			*/	^
            ^< ^< 0 0 Line [i++	/*				  show pixel and read new Y value	*/	^
            ; ^<		/*				ELSE drop the 1000			*/	^
         )			/*				ENDIF					*/	^
         4000 ^>? ^< : ^<	/*				Yn LSS 4000? drop 4000 and go back	*/	^
      ) ^<			/*			     WHILE Y value LSS 4000			*/	^
   )				/*			  ENDPROC Poly					*/	^
				/*									*/	^
   Label(			/* "A MESSAGE"		  PROC Label: Show a string as text		*/	^
      (  movc+0?		/* " MESSAGE" 'A'	     WHILE there is another char: 'A' = 65	*/	^
         Poly			/* " MESSAGE"			draw it with Poly			*/	^
         :			/*				go back for another char		*/	^
      ) ^<			/*			     ENDWHILE					*/	^
   )				/* 			  ENDPROC Label					*/	^
   "No format"			/* Start of main program		*/	^
   font SetFont			/* Load the font			*/	^
   %mulY% %mulX% SetSizes	/* Initialize font size scale factors	*/	^
   CURS ]0 ^< ]1 ^<		/* Initialize turtle position		*/	^
   %color% ]2 SF0		/* Initialize color and penDown		*/	^
   "%param%" Label		/* Show the message			*/

exit /B

@end

// Reactivate original command window after a graphic is drawn
new ActiveXObject("WScript.Shell").AppActivate(WScript.Arguments(0))
