;	printfStorage.asm: Data Definition Module for integer and floating point storage registers
;	                   and for strings and RPN stack data storage area in printf.exe application

;	Antonio Perez Ayala
;	version 1: 2023/01/01

	.486					;create 32 bit code
	.model flat, stdcall			;32 bit memory model
	option casemap :none			;case sensitive


	;Declaration of names exported to other modules
	PUBLIC		IREGS, IstorageI, Istorage,  FREGS, Fstorage,  CHARS
	PUBLIC		MAX_FORMAT, formatStack,  MAX_CMDSTDIN, cmdStdinStack,  NAMEDBLOCK_LEN, namedBlockName, namedBlockEntry
	PUBLIC		STACK_LEN, caseConv


	;These are the importing declarations in the other (base) file:

;	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;storage registers are declared in printfStorage.asm module 
;	EXTERN		IREGS:ABS, IstorageI:DWORD, Istorage:DWORD	;number/area of Integer storage registers
;	EXTERN		FREGS:ABS, Fstorage:REAL8			;number/area of Floating Point storage registers
;	EXTERN		CHARS:ABS					;size of memory block for string data area
;	EXTERN		MAX_FORMAT:ABS, formatStack:DWORD		;number/area of nested FMT{ marks
;	EXTERN		MAX_CMDSTDIN:ABS, cmdStdinStack:DWORD		;number/area of nested CMD's (redirected Stdin streams)
;	EXTERN		NAMEDBLOCK_LEN:ABS, namedBlockName:DWORD, namedBlockEntry:DWORD	;number/areas for defined subroutines
;	EXTERN		STACK_LEN:ABS					;size of memory block for "our stack" (printf.exe parameters)
;	EXTERN		caseConv:WORD					;case letters conversion table for CASE function
;	;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


	.data?					;Uninitialized data segment that don't take space in the .exe

	IREGS		EQU	10000		;Number of Integer storage registers
	FREGS		EQU	20		;Number of Floating Point storage registers
	CHARS		EQU	10 * 1024	;Size of memory block for string data = 10 KB
			;
	IstorageI	DD	?		;Index (i) integer register, *MUST BE PLACED* here! before Istorage
	Istorage	DD	IREGS DUP (?)	;Integer storage registers
	Fstorage	REAL8	FREGS DUP (?)	;Floating Point storage registers
			;
			;				;Reserve/Free data area for printf formats via: FMT{ ... FMT} delimiters
	MAX_FORMAT	EQU	40			;- Maximum nested FMT{ marks * 2
	formatStack	DD	MAX_FORMAT DUP (?)	;- Stack for stored FMT marks (comprised of Stack pointer and Color pairs)
			;
			;				;Nested management of Stdin FILE * streams via: CMD..IN? (EOF)
	MAX_CMDSTDIN	EQU	20			;- Maximum redirected Stdin's (nested CMD asynchronous functions)
	cmdStdinStack	DD	MAX_CMDSTDIN DUP (?)	;- Stack for redirected CMD Stdin streams
			;
	NAMEDBLOCK_LEN	EQU	50			;Allowed number of "namedBlocks" (subroutines)
	namedBlockName	DD	NAMEDBLOCK_LEN DUP (?)	;- array for "namedBlock" names
	namedBlockEntry	DD	NAMEDBLOCK_LEN DUP (?)	;- array for "namedBlock" entry points
			;
	STACK_LEN	EQU	2 * 1024		;size of memory block for "our stack" (parameters of printf.exe program)

	.data

	;Next table define upper/lower case letter conversions: there are 128 values specifying the characters in 128-255 range
	;A positive value indicate convert uppercase into lowercase. A negative value indicate the opposite conversion

		;4: 128  Ç    ü    é    â    ä    à    å    ç    ê    ë    è    ï    î    ì    Ä    Å	143
caseConv	DW	135,-154,-144,-182,-142,-183,-143,-128,-210,-211,-212,-216,-215,-222, 132, 134
		;   144  É    æ    Æ    ô    ö    ò    û    ù    ÿ    Ö    Ü    ø    £    Ø    ×    ƒ	159
		DW	130,-146, 145,-226,-153,-227,-234,-235,   0, 148, 129,-157,   0, 155,   0,   0
		;5: 160  á    í    ó    ú    ñ    Ñ    ª    º    ¿    ®    ¬    ½    ¼    ¡    «    »	175
		DW     -181,-214,-224,-233,-165, 164,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0
		;   176  ░    ▒    ▓    │    ┤    Á    Â    À    ©    ╣    ║    ╗    ╝    ¢    ¥    ┐	191
		DW	  0,   0,   0,   0,   0, 160, 131, 133,   0,   0,   0,   0,   0,   0,   0,   0
		;6: 192  └    ┴    ┬    ├    ─    ┼    ã    Ã    ╚    ╔    ╩    ╦    ╠    ═    ╬    ¤	207
		DW	  0,   0,   0,   0,   0,   0,-199, 198,   0,   0,   0,   0,   0,   0,   0,   0
		;   208  ð    Ð    Ê    Ë    È    ı    Í    Î    Ï    ┘    ┌    █    ▄    ¦    Ì    ▀	223
		DW	  0,   0, 136, 137, 138,   0, 161, 140, 139,   0,   0,   0,   0,   0, 141,   0
		;7: 224  Ó    ß    Ô    Ò    õ    Õ    µ    þ    Þ    Ú    Û    Ù    ý    Ý    ¯    ´	239
		DW	162,   0, 147, 149,-229, 228,   0, 132,-131, 163, 150, 151,-237, 236,   0,   0
		;   240  ­    ±    ‗    ¾    ¶    §    ÷    ¸    °    ¨    ·    ¹    ³    ²    ■     	255
		DW	  0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0,   0

	end
